﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.Drawing.Drawing2D;
using System.IO;
using System.Drawing.Imaging;

namespace howto_sized_oval_image
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private Bitmap OriginalImage = null;
        private Rectangle OvalRect =
            new Rectangle(0, 0, 200, 250);

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (ofdFile.ShowDialog() == DialogResult.OK)
            {
                OriginalImage = new Bitmap(ofdFile.FileName);
                picImage.Image = new Bitmap(ofdFile.FileName);
                saveAsToolStripMenuItem.Enabled = true;
                picImage.Visible = true;

                DrawImage();
            }
        }

        private void saveAsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (sfdFile.ShowDialog() == DialogResult.OK)
            {
                using (Bitmap bm = MakeOvalImage(OvalRect))
                {
                    SaveImage(bm, sfdFile.FileName);
                }
            }
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        // Select an elliptical area.
        private bool Dragging = false;
        private Point MousePos;
        private void picImage_MouseDown(object sender, MouseEventArgs e)
        {
            Dragging = true;
            MousePos = e.Location;
        }

        private void picImage_MouseMove(object sender, MouseEventArgs e)
        {
            if (!Dragging) return;
            OvalRect.X += e.Location.X - MousePos.X;
            OvalRect.Y += e.Location.Y - MousePos.Y;
            MousePos = e.Location;

            DrawImage();
        }

        private void picImage_MouseUp(object sender, MouseEventArgs e)
        {
            Dragging = false;
        }

        private void DrawImage()
        {
            Bitmap bm = new Bitmap(OriginalImage);
            using (Graphics gr = Graphics.FromImage(bm))
            {
                gr.SmoothingMode = SmoothingMode.AntiAlias;
                gr.DrawEllipse(Pens.Magenta, OvalRect);
            }
            picImage.Image = bm;
        }

        private Bitmap MakeOvalImage(Rectangle rect)
        {
            Bitmap bm = new Bitmap(rect.Width, rect.Height);

            using (Graphics gr = Graphics.FromImage(bm))
            {
                gr.SmoothingMode = SmoothingMode.AntiAlias;
                gr.Clear(Color.Transparent);
                using (TextureBrush brush =
                    new TextureBrush(OriginalImage, rect))
                {
                    rect.X = 0;
                    rect.Y = 0;
                    gr.FillEllipse(brush, rect);
                }
            }
            return bm;
        }

        private void txtSize_TextChanged(object sender, EventArgs e)
        {
            int width, height;
            if (int.TryParse(txtWidth.Text, out width))
                OvalRect.Width = width;
            if (int.TryParse(txtHeight.Text, out height))
                OvalRect.Height = height;

            DrawImage();
        }

        // Save the file with the appropriate format.
        public void SaveImage(Image image, string filename)
        {
            string extension = Path.GetExtension(filename);
            switch (extension.ToLower())
            {
                case ".bmp":
                    image.Save(filename, ImageFormat.Bmp);
                    break;
                case ".exif":
                    image.Save(filename, ImageFormat.Exif);
                    break;
                case ".gif":
                    image.Save(filename, ImageFormat.Gif);
                    break;
                case ".jpg":
                case ".jpeg":
                    image.Save(filename, ImageFormat.Jpeg);
                    break;
                case ".png":
                    image.Save(filename, ImageFormat.Png);
                    break;
                case ".tif":
                case ".tiff":
                    image.Save(filename, ImageFormat.Tiff);
                    break;
                default:
                    throw new NotSupportedException(
                        "Unknown file extension " + extension);
            }
        }
    }
}
